import { __spread, __values, __decorate, __metadata } from 'tslib';
import { Injectable, NgZone, isDevMode, Input, Output, Directive, ElementRef, EventEmitter, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';

var KeyboardShortcutListener = /** @class */ (function () {
    function KeyboardShortcutListener(listenerConstructorObject, blackListedKeyboardShortcutChecker) {
        // assign properties based on constructor and defaults
        var defaultListenerOptions = {
            ignoreEventsFromInputElement: false,
            omitFromHelp: false,
            passToLowerPriorities: true,
            priority: 0
        };
        Object.assign(this, defaultListenerOptions, listenerConstructorObject);
        // assign mapped keyboard shortcut combination
        this.mappedKeyboardShortcutCombination = mapKeyboardShortcutCombination(this.keyBinding);
        // assign display shortcut combination
        var copyOfKeyBinding = __spread(this.keyBinding);
        copyOfKeyBinding.sort(function (a, b) {
            if (a.length > 1 && b.length === 1) {
                return -1;
            }
            else if (a.length === 1 && b.length > 1) {
                return 1;
            }
            else if (a > b) {
                return 1;
            }
            else if (a < b) {
                return -1;
            }
            else {
                return 0;
            }
        });
        this.displayShortcutCombination = copyOfKeyBinding
            .map(function (kb) { return kb.replace(/^\w/, function (c) { return c.toUpperCase(); }); })
            .join(' + ');
        // check vs blacklist
        if (blackListedKeyboardShortcutChecker) {
            blackListedKeyboardShortcutChecker.check(this);
        }
    }
    return KeyboardShortcutListener;
}());
/**
 * @enum this allows you to map special keys into your Keyboard Combination
 */
var KeyboardKeys;
(function (KeyboardKeys) {
    KeyboardKeys["Ctrl"] = "ctrl";
    KeyboardKeys["Alt"] = "alt";
    KeyboardKeys["Shift"] = "shift";
    KeyboardKeys["Escape"] = "escape";
})(KeyboardKeys || (KeyboardKeys = {}));
/**
 * @param bindings takes in the shortcut from KeyboardShortcutCombination which is an array of strings
 * @return {string}
 */
function mapKeyboardShortcutCombination(bindings) {
    return JSON.stringify(bindings.map(function (key) { return key.toLowerCase(); }).sort());
}
// Map to normalized keys across different browser implementations.
// https://github.com/angular/angular/blob/5.0.5/packages/platform-browser/src/browser/browser_adapter.ts#L25-L42
// tslint:disable:object-literal-sort-keys
/**
 * @constant
 * Map to convert some key or keyIdentifier values to what will be returned by getEventKey
 * This is also used to help browser compatibility and to match the W3C standard
 */
var KEY_MAP = {
    '\b': 'backspace',
    '\t': 'tab',
    '\x1B': 'escape',
    '\x7F': 'delete',
    Del: 'delete',
    Esc: 'escape',
    Left: 'arrowleft',
    Right: 'arrowright',
    Up: 'arrowup',
    Down: 'arrowdown',
    Menu: 'contextMenu',
    Scroll: 'scrolllock',
    Win: 'os',
    ' ': 'space',
    '.': 'dot'
};
// tslint:enable:object-literal-sort-keys

var blacklistedKeyCombinations = [
    [KeyboardKeys.Ctrl, 'd'],
    [KeyboardKeys.Ctrl, 'f'],
    [KeyboardKeys.Ctrl, 'g'],
    [KeyboardKeys.Ctrl, KeyboardKeys.Shift, 'g'],
    [KeyboardKeys.Ctrl, 'h'],
    [KeyboardKeys.Ctrl, 'j'],
    [KeyboardKeys.Ctrl, 'o'],
    [KeyboardKeys.Ctrl, 's'],
    [KeyboardKeys.Ctrl, 'p'],
    [KeyboardKeys.Ctrl, 'u'],
    [KeyboardKeys.Ctrl, 'n'],
    [KeyboardKeys.Ctrl, 't'],
    [KeyboardKeys.Ctrl, KeyboardKeys.Shift, 't'],
    [KeyboardKeys.Ctrl, 'c'],
    [KeyboardKeys.Ctrl, 'x'],
    [KeyboardKeys.Ctrl, 'v'],
    [KeyboardKeys.Alt, 'f'],
    [KeyboardKeys.Ctrl, '0'],
    [KeyboardKeys.Ctrl, '1'],
    [KeyboardKeys.Ctrl, '2'],
    [KeyboardKeys.Ctrl, '3'],
    [KeyboardKeys.Ctrl, '4'],
    [KeyboardKeys.Ctrl, '5'],
    [KeyboardKeys.Ctrl, '6'],
    [KeyboardKeys.Ctrl, '7'],
    [KeyboardKeys.Ctrl, '8'],
    [KeyboardKeys.Ctrl, '9'],
    [KeyboardKeys.Shift, '1'],
    [KeyboardKeys.Shift, '2'],
    [KeyboardKeys.Shift, '3'],
    [KeyboardKeys.Shift, '4'],
    [KeyboardKeys.Shift, '5'],
    [KeyboardKeys.Shift, '6'],
    [KeyboardKeys.Shift, '7'],
    [KeyboardKeys.Shift, '8'],
    [KeyboardKeys.Shift, '9'],
    [KeyboardKeys.Shift, '0'],
    [KeyboardKeys.Escape],
    [KeyboardKeys.Ctrl, 'tab'],
    [KeyboardKeys.Ctrl, KeyboardKeys.Shift, 'tab'],
    [KeyboardKeys.Ctrl, 'w'],
    [KeyboardKeys.Alt, '4'],
    [KeyboardKeys.Alt, 'arrowleft'],
    [KeyboardKeys.Alt, 'backspace'],
    [KeyboardKeys.Ctrl, KeyboardKeys.Shift, 'delete'] // open the clear browsing history window
];
var BlackListedKeyboardShortcutChecker = /** @class */ (function () {
    function BlackListedKeyboardShortcutChecker() {
        this.blackListedKeyBindings = blacklistedKeyCombinations;
        this.mappedBlackListedKeyBindings = this.blackListedKeyBindings.map(function (blkb) { return mapKeyboardShortcutCombination(blkb); });
    }
    BlackListedKeyboardShortcutChecker.prototype.check = function (listener) {
        var e_1, _a;
        try {
            // this is where the test was deleted
            for (var _b = __values(this.mappedBlackListedKeyBindings), _c = _b.next(); !_c.done; _c = _b.next()) {
                var blkb = _c.value;
                if (blkb === listener.mappedKeyboardShortcutCombination) {
                    var warningMessage = "Keyboard Shortcut [" + listener.displayShortcutCombination + "] is blacklisted as a common keyboard shortcut.  Consider changing your keybinding for this shortcut.";
                    console.warn(warningMessage);
                    return;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    return BlackListedKeyboardShortcutChecker;
}());

var KeyboardShortcutService = /** @class */ (function () {
    function KeyboardShortcutService(zone) {
        var _this = this;
        this._listeners = [];
        this.zone = zone;
        // Since we're going to create a root event-handler for the keydown event, we're
        // gonna do this outside of the NgZone. This way, we're not constantly triggering
        // change-detection for every key event - we'll only re-enter the Angular Zone
        // when we have an event that is actually being consumed by one of our components.
        this.zone.runOutsideAngular(function () {
            window.addEventListener('keydown', _this.handleKeyboardEvent.bind(_this));
        });
        // we only use this if isDevMode
        if (isDevMode()) {
            this.blackListedKeyboardShortcutChecker = new BlackListedKeyboardShortcutChecker();
        }
    }
    Object.defineProperty(KeyboardShortcutService.prototype, "listeners_read_only", {
        /**
         * @readonly
         * @description A list of all available shortcut listeners
         * @property    {IKeyboardShortcutListener[]}
         */
        get: function () {
            return JSON.parse(JSON.stringify(this._listeners));
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @description                     Creates a listenerHandle using imported interface
     * @param listenerConstructorObject Interface that contains a KeyboardShortcutHandler
     * @returns                         an IListenerHandle
     */
    KeyboardShortcutService.prototype.listen = function (listenerConstructorObject) {
        var listenerHandle = this.addListener(listenerConstructorObject);
        return listenerHandle;
    };
    KeyboardShortcutService.prototype.addListener = function (listenerConstructorObject) {
        var _this = this;
        var listener = new KeyboardShortcutListener(listenerConstructorObject, this.blackListedKeyboardShortcutChecker);
        var listenerHandle = {
            remove: function () {
                _this.removeListener(listener);
            }
        };
        this._listeners.push(listener);
        this._listeners.sort(function (a, b) {
            if (a.priority < b.priority) {
                return 1;
            }
            else if (a.priority > b.priority) {
                return -1;
            }
            else {
                return 0;
            }
        });
        return listenerHandle;
    };
    // get the keyCombination from the given event
    KeyboardShortcutService.prototype.getKeyFromEvent = function (event) {
        var keyIdentifier = 'keyIdentifier';
        var key = event.key || event[keyIdentifier] || 'Unidentified';
        if (key.startsWith('U+')) {
            key = String.fromCharCode(parseInt(key.slice(2), 16));
        }
        var parts = [(KEY_MAP[key] || key)];
        if (event.altKey) {
            parts.push(KeyboardKeys.Alt);
        }
        if (event.ctrlKey) {
            parts.push(KeyboardKeys.Ctrl);
        }
        if (event.shiftKey) {
            parts.push(KeyboardKeys.Shift);
        }
        return mapKeyboardShortcutCombination(parts);
    };
    // handle the keyboard events for the root handler (and delegate to the listeners).
    KeyboardShortcutService.prototype.handleKeyboardEvent = function (event) {
        var e_1, _a;
        var keyCombination = this.getKeyFromEvent(event);
        var isInputEvent = this.isEventFromInput(event);
        var handler;
        try {
            // Iterate over the listeners in DESCENDING priority order.
            for (var _b = __values(this._listeners), _c = _b.next(); !_c.done; _c = _b.next()) {
                var listener = _c.value;
                if (listener.mappedKeyboardShortcutCombination === keyCombination) {
                    handler = listener.handler;
                    // Execute handler if this is NOT an input event that we need to ignore.
                    if (!isInputEvent || !listener.ignoreEventsFromInputElement) {
                        // Right now, we're executing outside of the NgZone. As such, we
                        // have to re-enter the NgZone so that we can hook back into change-
                        // detection. Plus, this will also catch errors and propagate them
                        // through application properly.
                        this.zone.runGuarded(function () {
                            return handler(event);
                        });
                        // If the handler returned an explicit False, we're going to treat
                        // this listener as Terminal, regardless of the original settings.
                        if (!listener.passToLowerPriorities) {
                            return;
                        }
                        else {
                            continue;
                        }
                    }
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    /**
     * @description Used for testing, Sends a keyboard event
     * @param event a keyboard event
     */
    KeyboardShortcutService.prototype.sendKeyboardEventToHandler = function (event) {
        this.handleKeyboardEvent(event);
    };
    // determine if the given event originated from a form input element.
    KeyboardShortcutService.prototype.isEventFromInput = function (event) {
        if (event.target instanceof Node) {
            switch (event.target.nodeName) {
                case 'INPUT':
                case 'SELECT':
                case 'TEXTAREA':
                    return true;
                default:
                    return false;
            }
        }
        return false;
    };
    // remove the given listener from the internal collection.
    KeyboardShortcutService.prototype.removeListener = function (listenerToRemove) {
        this._listeners = this._listeners.filter(function (listener) {
            return listener !== listenerToRemove;
        });
    };
    KeyboardShortcutService = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [NgZone])
    ], KeyboardShortcutService);
    return KeyboardShortcutService;
}());

var KeyboardShortcutDirective = /** @class */ (function () {
    function KeyboardShortcutDirective(elRef, keyboardShortcutService) {
        this.elRef = elRef;
        this.keyboardShortcutService = keyboardShortcutService;
        this.fireClickEventOnKeyboardShortcut = true;
        this.onKeyboardShortcut = new EventEmitter();
    }
    KeyboardShortcutDirective.prototype.ngOnInit = function () {
        if (!this.keyboardShortcut) {
            return;
        }
        var listenerConstructor = {};
        Object.assign(listenerConstructor, { handler: this.keyboardShortcutHandler.bind(this) }, this.keyboardShortcut);
        this.listener = this.keyboardShortcutService.listen(listenerConstructor);
    };
    KeyboardShortcutDirective.prototype.keyboardShortcutHandler = function (event) {
        if (this.fireClickEventOnKeyboardShortcut) {
            if (this.elRef &&
                this.elRef.nativeElement &&
                this.elRef.nativeElement.click) {
                this.elRef.nativeElement.click();
            }
        }
        this.onKeyboardShortcut.emit(event);
    };
    KeyboardShortcutDirective.prototype.ngOnDestroy = function () {
        if (this.listener) {
            this.listener.remove();
        }
    };
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], KeyboardShortcutDirective.prototype, "keyboardShortcut", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], KeyboardShortcutDirective.prototype, "fireClickEventOnKeyboardShortcut", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], KeyboardShortcutDirective.prototype, "onKeyboardShortcut", void 0);
    KeyboardShortcutDirective = __decorate([
        Directive({
            selector: '[keyboardShortcut]'
        }),
        __metadata("design:paramtypes", [ElementRef,
            KeyboardShortcutService])
    ], KeyboardShortcutDirective);
    return KeyboardShortcutDirective;
}());

var NgxKeyboardShortcutModule = /** @class */ (function () {
    function NgxKeyboardShortcutModule() {
    }
    NgxKeyboardShortcutModule_1 = NgxKeyboardShortcutModule;
    NgxKeyboardShortcutModule.forRoot = function () {
        return {
            ngModule: NgxKeyboardShortcutModule_1,
            providers: [
                KeyboardShortcutService
            ]
        };
    };
    var NgxKeyboardShortcutModule_1;
    NgxKeyboardShortcutModule = NgxKeyboardShortcutModule_1 = __decorate([
        NgModule({
            declarations: [KeyboardShortcutDirective],
            exports: [KeyboardShortcutDirective],
            imports: [CommonModule]
        })
    ], NgxKeyboardShortcutModule);
    return NgxKeyboardShortcutModule;
}());

export { KEY_MAP, KeyboardKeys, KeyboardShortcutDirective, KeyboardShortcutListener, KeyboardShortcutService, NgxKeyboardShortcutModule, blacklistedKeyCombinations, mapKeyboardShortcutCombination, BlackListedKeyboardShortcutChecker as ɵa };
//# sourceMappingURL=ngx-keyboard-shortcuts.js.map
